this.name      = 'eq_enemy_targeter';
this.author    = 'Andrey Belov';
this.copyright = '2013 Andrey Belov aka timer [ TiNe Corp. ]';
this.licence   = 'CC BY-NC-SA 3.0'; // see http://creativecommons.org/licenses/by-nc-sa/3.0/ for more info.
this.version   = '0.5';


this._mode = 0;

this.activated = function() {
	if ( player.ship.equipmentStatus('EQ_ENEMY_TARGETER_UPGRADE') === 'EQUIPMENT_OK' ) {
		this.set_next_target();
	}
};

this.mode = function() {
	if ( player.ship.equipmentStatus('EQ_ENEMY_TARGETER_UPGRADE') !== 'EQUIPMENT_OK' ) {
		return;
	}
	this._mode++;
	if ( this._mode > 2 ) {
		this._mode = 0;
	}
	player.consoleMessage( 'TSU mode: ' + ['hostile/outlaw','neutral','cargo'][this._mode], 1 );
};

this.set_next_target = function() {
	var targets = [];

	if ( 0 === this._mode ) {
		targets = this.get_hostile_ships();

		if ( targets.length < 1 ) {
			targets = this.get_outlaw_ships();
		}
	}
	else if ( 1 === this._mode ) {
		targets = this.get_pacific_ships();
	}
	else if ( 2 === this._mode ) {
		targets = this.get_cargo_pods();
	}

	var len = targets.length - 1;

	if ( len < 0 ) {
		return;
	}

	this.sort_by_distance(targets);

	var ptn = 0;
	if ( player.ship.target ) {
		var enemy_targeter = worldScripts['enemy_targeter_main'];
		ptn = enemy_targeter.get_previous_target_num();
		ptn = ptn >= len ? 0 : ptn + 1;
		enemy_targeter.set_previous_target_num(ptn);
	}
	var next_target = targets[ptn];

	if ( player.ship.target && player.ship.target === next_target ) {
		return;
	}
	player.ship.target = next_target;
};

this.get_hostile_ships = function() {
	if ( ! player.alertHostiles ) {
		return [];
	}
	return system.filteredEntities( this, function (e) { return ( e.isShip && e.hasHostileTarget && e.target === player.ship ); }, player.ship, player.ship.scannerRange );
};

this.get_outlaw_ships = function() {
	if ( 2 > player.alertCondition ) {
		return [];
	}
	return system.filteredEntities( this, function (e) { return ( e.scanClass === 'CLASS_NEUTRAL' && e.bounty !== 0 ); }, player.ship, player.ship.scannerRange );
};

this.get_pacific_ships = function() {
	if ( 2 > player.alertCondition ) {
		return [];
	}
	return system.filteredEntities( this, function (e) { return ( e.scanClass === 'CLASS_NEUTRAL' && e.bounty === 0 && e.target !== player.ship ); }, player.ship, player.ship.scannerRange );
};

this.get_cargo_pods = function() {
	return system.filteredEntities( this, function (e) { return ( e.scanClass === 'CLASS_CARGO' ); }, player.ship, player.ship.scannerRange );
};

this.sort_by_distance = function(targets) {
	var cache = {};
	// sort
	targets.sort( function sortByDist(a,b) {
		// cache distance
		//var ai=a.ID,bi=b.ID; // work only with my patch with universalID getter :(
		var ai=a.entityPersonality,bi=b.entityPersonality; // use some "uniq" ident
		cache[ai] = cache[ai] || a.position.distanceTo(player.ship.position);
		cache[bi] = cache[bi] || b.position.distanceTo(player.ship.position);
		return ((cache[ai]<cache[bi])?-1:((cache[ai]>cache[bi])?1:0));
	} );
};
